# -*- coding: euc-jp -*-

import pygame
import time
from pygame.locals import *

class DemoException(Exception):
    pass

class QuitException(DemoException):
    pass

class NextException(DemoException):
    pass

class PrevException(DemoException):
    pass

class TimeoutException(DemoException):
    pass

BLACK = (0, 0, 0, 255)

def calldemo(subr):
    try:
        subr()
    except QuitException:
        sys.exit(0)

def demomain(slidedrawers):
    idx = 0
    l = len(slidedrawers)
    while True:
        try:
            drawer = slidedrawers[idx]
            drawer()
            wait()
        except QuitException:
            break
        except PrevException:
            idx -= 1
        except  NextException:
            idx += 1
        idx = (idx + l)%l

TIMEOUTEV = USEREVENT + 1

def isQuitEvent(e):
    if e.type == QUIT:
        return True
    elif e.type == KEYDOWN and ( e.key == K_ESCAPE or e.key == K_q ):
        return True
    return False

def wait(millisecs=None):
    if millisecs is not None:
        pygame.time.set_timer(TIMEOUTEV, millisecs)
    else:
        pygame.time.set_timer(TIMEOUTEV, -1)
    while True:
        e = pygame.event.wait()
        if e.type == TIMEOUTEV:
            pygame.time.set_timer(TIMEOUTEV, -1)
            break
        elif isQuitEvent(e):
            raise QuitException()
        elif e.type == KEYDOWN and (e.key in (K_SPACE, K_n, K_RIGHT)) or e.type == MOUSEBUTTONDOWN:
            raise NextException()
        elif e.type == KEYDOWN and (e.key in (K_BACKSPACE, K_p, K_LEFT)):
            raise PrevException()

def waitsec(secs=None):
    if secs is not None:
        secs = int(secs*1000)
    wait(secs)

def fullscreenwait():
    time.sleep(3)


defaultFont = None
defaultFontSize = 20

def getDefaultFont():
    global defaultFont
    if defaultFont is None:
        defaultFont = pygame.font.Font(None, defaultFontSize)
    return defaultFont

class NewLine:
    pass

def renderText(surf, specs, x=0, y=0, color=None, font=None):
    bx, by = x, y
    x, y = 0, 0
    if color is None:
        color = BLACK
    if font is None:
        font = getDefaultFont()
    maxHeight = 0
    justRendered = False
    sh = surf.get_height()
    for spec in specs:
        if y+by > sh:
            break
        justRenderedNext = False
        if type(spec) == tuple:
            color = spec
        elif type(spec) == pygame.font.Font:
            font = spec
        else:
            if justRendered:
                if maxHeight == 0:
                    dy = font.get_height()
                else:
                    dy = maxHeight
                y += dy
                x = 0
            s = font.render(spec, 1, color)
            surf.blit(s, (x+bx, y+by))
#             strips.append((x, y, s))
            x += s.get_width()
            maxHeight = max(maxHeight, s.get_height())
#             maxx = max(maxx, x)
#             maxy = max(maxy, y+maxHeight)
            justRenderedNext = True
        justRendered = justRenderedNext


if __name__ == '__main__':
    def test():
        pygame.display.init()
        pygame.font.init()
        f = pygame.font.Font("gothic.ttf", 50)
        scr = pygame.display.set_mode((640, 480))
        renderText(scr, [pygame.color.Color("white"), "Hello, ", pygame.color.Color("blue"), "world!", "", pygame.color.Color("red"), u"̵ʩ"], font=f, x=100, y=300)
        pygame.display.flip()
        wait()
    test()

