import pygame
from pygame.locals import *
import random
import math
import sys, os, re
import hotshot
import Numeric
import time

import somcore

import cPickle
import demoutils

import codecs
import getdist

VECCOES = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 100, 100, 100, 1, 1, 1, 1.5]

HEADERSTOSHOW = ('from', 'to', 'cc', 'subject')

WIDTH = 64
HEIGHT = 46
MAG = 16

MAILFONTNAME = "mincho.ttf"
MAILFONTSIZE = 20

STATH = 32

DTHRESH = 0.5**2

BLACK = (0, 0, 0, 255)
WHITE = (255, 255, 255, 255)
RED = (255, 0, 0, 255)
STATUSRECT = pygame.Rect(0, HEIGHT*MAG, WIDTH*MAG, STATH)
FONT = None

#autoProceedF = True
autoProceedF = False
nonDisplayMode = False
#viewMappingMode = False
viewMappingMode = True
mapimg = None

origMessageDir = "origMesgs"

def xo(x):
    return min(int(x), 255)

def getcolor(v):
#    return (int(v[6]*128/VECCOES[6]), xo((v[7]*v[7]+v[8]*v[8]+v[9]*v[9])*192), int(v[16]*256/VECCOES[16]))
#    return (int(v[6]*128/VECCOES[6]), xo((v[10]*v[10]+v[11]*v[11]+v[12]*v[12])*256), int(v[16]*256/VECCOES[16]))
    return (int(v[6]*128/VECCOES[6]), xo((v[10]+v[11]+v[12])*128), int(v[16]*256/VECCOES[16]))

def pyRandomize(ary):
    pyRandomizeAux(ary, len(Numeric.shape(ary)))

def pyRandomizeAux(ary, levelsLeft):
    if levelsLeft == 1:
        for i in xrange(len(ary)):
            ary[i] = random.random()
    else:
        for i in xrange(len(ary)):
            pyRandomizeAux(ary[i], levelsLeft-1)

def norm2(a, b):
    s = 0
    lim = len(a)
    i = 0
    while i < lim:
        d = a[i]-b[i]
        s += d*d
        i += 1
    return s

class SOM(object):
    def __init__(self):
        pass

    def initNew(self, w, h, dim):
        self.initM(w, h, dim)
        self.renewGeom()

    def initLoad(self, fn):
        self.M = Numeric.load(file(fn, "rb"))
        self.renewGeom()

    def saveTo(self, fn):
        Numeric.dump(self.M, file(fn, "wb"))

    def renewGeom(self):
        self.w, self.h, self.dim = self.M.shape
        self.wr = xrange(self.w)
        self.hr = xrange(self.h)
        self.dimr = xrange(self.dim)
        self.initWorks()

    def initM(self, w, h, dim):
        self.M = Numeric.zeros((w, h, dim), 'd')
        somcore.srandom(2)
        somcore.randomize(self.M)

    def initWorks(self):
        self.NM = Numeric.ones((self.w, self.h), 'd')
        self.SADISP = Numeric.zeros((self.w*MAG, self.h*MAG, 3), 'b')

    def learn(self, v, decayLevel):
        nearest = self.getNearest(v)
        self.update(nearest, v, decayLevel)
        return nearest

    def cUpdate(self, pos, v, decayLevel):
        (px, py) = pos
        d = self.decayFactor(decayLevel)
        nr = self.neighbourRadius(decayLevel)
#        somcore.update(self.M, Numeric.array(v, 'd'), px, py, d, nr)
        somcore.update(self.M, v, px, py, d, nr)

# stale interface
    def pyUpdate(self, pos, v):
        (px, py) = pos
        d = self.decayFactor()
        nr = self.neighbourRadius()
        for mx in self.wr:
            for my in self.hr:
                factor = self.distFactor(px, py, mx, my, nr) * d
                for i in self.dimr:
                    self.M[mx][my][i] += factor * (v[i] - self.M[mx][my][i])

    update = cUpdate

    # $B$$$$2C8:!#(B
    def decayFactor(self, decayLevel):
        return 0.1/math.log((decayLevel+1)*math.e)
#        return 0.05/math.log((self.decayLevel+1)*math.e)
#        return 0.02/math.log((self.decayLevel+1)*math.e)
#         df = 0.02/math.log((self.decayLevel+1+((self.cnt*1.0/(self.w*self.h)) % 1.0))*math.e)
#         print df
#         return df

    def distFactor(self, x1, y1, x2, y2, nr):
        tv = norm2((x1, y1), (x2, y2))

## gaussian
#         r = self.neighbourRadius()
#         rv = math.exp(-tv/(2*r*r))

## cylinder
        r2 = nr*nr
        if tv > r2:
            rv = 0
        else:
            rv = 1
        return rv

    # $B$3$l$^$?$$$$2C8:!#(B
    def neighbourRadius(self, decayLevel):
        nr = 1+(self.w+self.h)/10.0/(decayLevel+0.3)
        return nr

    def cGetNearest(self, v):
        return somcore.getWinner(self.M, v, self.NM)
#        return somcore.getWinner(self.M, Numeric.array(v, 'd'), self.NM)

    def pyGetNearest(self, v):
        nearest = None
        for y in xrange(self.h):
            for x in xrange(self.w):
                n2 = norm2(v, self.M[x][y])
                self.NM[x][y] = math.sqrt(n2)
                if nearest is None or n2 < nearestn2:
                    nearest = (x, y)
                    nearestn2 = n2
        return nearest

#    getNearest = pyGetNearest
    getNearest = cGetNearest

    # $B$;$a$FFs<!4X?t$G(B fitting $B$7$?$$$H$3$m$@$,!#(B
    def pyGetVecPos(self, v, pos=None):
        if pos is None:
            pos = self.getNearest(v)
        x, y = pos
        n2 = norm2(v, self.M[x][y])
        n = math.sqrt(n2)

        if x == 0 or x == self.w-1:
            dx = 0
        else:
            nm = math.sqrt(norm2(v, self.M[x-1][y]))-n
            np = math.sqrt(norm2(v, self.M[x+1][y]))-n
            if nm + np == 0:
                dx = 0
            else:
                dx = nm/(nm+np)-0.5

        if y == 0 or y == self.h-1:
            dy = 0
        else:
            nm = math.sqrt(norm2(v, self.M[x][y-1]))-n
            np = math.sqrt(norm2(v, self.M[x][y+1]))-n
            if nm + np == 0:
                dy = 0
            else:
                dy = nm/(nm+np)-0.5

        return x+dx, y+dy

    getVecPos = pyGetVecPos

    def __repr__(self):
        return "%sx%sx%s dL:%s\n%s" % (self.w, self.h, self.dim, self.decayLevel, self.m)

    def beforePickle(self):
        self.wr = None
        self.hr = None
        self.dimr = None
        self.SADISP = None
        self.NM = None

    def afterPickleOrLoad(self):
#        self.M = refreshArray(self.M)
        self.renewGeom()

# $BJV$jCM$O(B list or lists$B!#(B
def getMappingDensityMatrix(som, im):
    rv = []
    for y in xrange(len(som.M[0])):
        rv.append([0]*len(som.M))
    iw = im.get_width()
    ih = im.get_height()
    for iy in xrange(ih):
        for ix in xrange(iw):
            pix = im.get_at((ix, iy))
            tv = [v/256.0 for v in pix[0:3]]
            (tx, ty) = som.getNearest(tv)
            rv[ty][tx] += 1

    return rv

gcnt = 0
gim = None

def axy3toSimpleSurf(axy3):
    w = len(axy3)
    h = len(axy3[0])
    rv = pygame.Surface((w, h))
    pix = [0,0,0,255]
    for y in xrange(h):
        for x in xrange(w):
            for j in xrange(3):
                pix[j] = round(255*axy3[x][y][j])
            rv.set_at((x, y), pix)
    return rv

def vxy3toSimpleSurf(vxy3):
    w = len(vxy3[0])
    h = len(vxy3)
    rv = pygame.Surface((w, h))
    pix = [0,0,0,255]
    for y in xrange(h):
        for x in xrange(w):
            for j in xrange(3):
                pix[j] = round(255*vxy3[y][x][j])
            rv.set_at((x, y), pix)
    return rv

def refreshArray(x):
    tv = Numeric.zeros(x.shape, 'd')
    tv = x
    return tv
#    return Numeric.array(x, 'd')

# mail $B$K$d$dFC2=(B
class somuser(object):
    def __init__(self, m, vecsOrFiles, decayCoef=1):
        self.m = m
        if type(vecsOrFiles) == list:
            self.loadVecs(vecsOrFiles)
        else:
            self.vecs = vecsOrFiles
        self.vecsChanged()
        self.cnt = 0
        self.decayCoef = float(decayCoef)
        self.readFromHere = self.n  # perfect tense

    def vecsChanged(self):
        self.n = len(self.vecs)
        self.vr = xrange(self.n)
        self.mappings = Numeric.zeros((self.n, 2), 'd')
        self.zorder = range(self.n)
        self.staleMapping = True

    def loadVecs(self, files):
        n = len(files)
        tv = fn2vec(files[0])
        dim = len(tv)
        self.vecs = Numeric.zeros((n, len(tv)), 'd')
        for i in xrange(n):
            self.vecs[i] = fn2vec(files[i])
            if len(self.vecs[i]) != dim:
                raise RuntimeError("dimension mismatch! %s %s" % (len(self.vecs[i]), dim))
        self.vecFNs = files[:]
            
    def getVecIdxAt(self, x, y, w, h, unReadOnlyF=False):
        ct = 0
        for i in self.zorder:
            tx, ty = self.mappings[i]
            if tx >= x and tx <= x+w and ty >= y and ty <= y+h:
                if unReadOnlyF and ct >= self.readFromHere:
                    return -1
                return i
            ct += 1
        return -1

    def getDecay(self):
        return self.decayCoef * self.cnt / self.n
    
    def learn1(self):
        winner = self.m.learn(self.nextVec(), self.getDecay())
        self.cnt+=1
        self.staleMapping = True
        return winner

    def cCalcMappings(self, accurateFlag=False):
        somcore.calcMappings(self.vecs, self.m.M, self.mappings, accurateFlag)

    def pyCalcMappings(self):
        for i in self.vr:
            self.mappings[i] = self.m.getVecPos(self.vecs[i])

    doCalcMappings = cCalcMappings
    def calcMappings(self, accurateFlag=False):
        if self.staleMapping:
            self.doCalcMappings(accurateFlag)
        self.staleMapping = False

#    calcMappings = pyCalcMappings

    def statstr(self):
        return "%s  %s" % (self.cnt/self.n, self.cnt%self.n)

    def nextVec(self):
        idx = self.cnt % self.n
        return self.vecs[idx]

    def saveTo(self, fn):
        self.beforePickle()
        cPickle.dump(self, file(fn, "wb"))
        self.afterPickleOrLoad()

    def beforePickle(self):
        self.m.beforePickle()
        self.vr = None

    def afterPickleOrLoad(self):
        self.vr = xrange(self.n)
#        self.vecs = refreshArray(self.vecs)
#        self.mappings = refreshArray(self.mappings)
        self.m.afterPickleOrLoad()
        self.staleMapping = True

    def loadFrom(self, fn):
        me = cPickle.load(file(fn, "rb"))
        for k in me.__dict__:
            self.__dict__[k] = me.__dict__[k]
        self.afterPickleOrLoad()

    def fromFile(fn):
        me = cPickle.load(file(fn, "rb"))
        me.afterPickleOrLoad()
        return me

    fromFile = staticmethod(fromFile)

    def markRead(self, idx):
        z = self.zorder.index(idx)
        if z >= self.readFromHere:
            return
        self.zorder[z:-1] = self.zorder[z+1:]
        self.zorder[-1] = idx
        self.readFromHere -= 1

icons = None
iconFiles = ["mailicon-small.png", "mailicon-small-read.png"]
iconTrans = [0.85, 0.4]
icondx = 0
icondy = 0
icw = 0
ich = 0


def setupIcons():
    global icons, icondx, icondy, icw, ich

    icons = []
    for i in xrange(len(iconFiles)):
        fn = iconFiles[i]
        trans = iconTrans[i]
        ic = pygame.image.load(fn).convert_alpha()
        makeTrans(ic, trans)
        icons.append(ic)

    icw = icons[0].get_width()
    ich = icons[0].get_height()
    icondx = MAG/2.0 - icw/2.0
    icondy = MAG/2.0 - ich/2.0

def xblit(surf, a, b):
    surf.blit(a, b)

# calcMappings $B$O8F$P$l$?8e$H2>Dj(B
def showMailIcons(surf, mu):
    if icons is None:
        setupIcons()

    icon = icons[1]
    for idx in xrange(mu.n-1, -1, -1):
        if idx < mu.readFromHere:
            icon = icons[0]
        i = mu.zorder[idx]
        x, y = mu.mappings[i]
        try:
            xblit(surf, icon, (x*MAG+icondx, y*MAG+icondy))
        except:
            print "blit failure at %s %s" % (x, y)
            print idx
            print mu.vecs[i]
#        surf.blit(icon, (x*MAG+icondx, y*MAG+icondy))
#        surf.blit(icons[iconNo], (x*MAG+icondx, y*MAG+icondy))

def flip():
    pygame.display.flip()

def drawA(surf, mu, mesg=None, winner=None, flashF=True):
    if mesg is None:
        mesg = mu.statstr()
    draw(surf, mu, mesg=mesg, winner=winner, flashF=flashF)

def proceed(surf, mu):
    c = getcolor(mu.nextVec())
    winner = mu.learn1()
    drawA(surf, mu, winner=winner)
    surf.fill(c, pygame.Rect(WIDTH*MAG-15, HEIGHT*MAG+STATH-15, 10, 10))
    flip()

class QuitMessage(Exception):
    pass

def mainloop(surf, mu, limit=None):
    global autoProceedF

    cnt = 0
    try:
        while True:
            if autoProceedF:
                for e in pygame.event.get():
                    processEvent(e, surf, mu)
                proceed(surf, mu)
                cnt += 1
                if limit is not None and cnt >= limit:
                    break
            else:
                e = pygame.event.wait()
                processEvent(e, surf, mu)
    except QuitMessage:
        pass

gMailShowing = False

def doFlash(x, y, w, h, mu, surf):
    idx = mu.getVecIdxAt(x, y, w, h, unReadOnlyF=True)
    if idx >= 0:
        mu.m.getNearest(mu.vecs[idx])
        drawA(surf, mu, flashF=True)
        flip()
        return True
    return False

def showMessageAt(x, y, w, h, mu, surf):
    global gMailShowing

    idx = mu.getVecIdxAt(x, y, w, h)
    if idx >= 0:
#         mu.m.getNearest(mu.vecs[idx])
#         drawA(surf, mu, flashF=True)
        showMessageVecFn(mu.vecFNs[idx], surf)
        gMailShowing = True
        mu.markRead(idx)
        flip()

def showMessageVecFn(vecfn, surf):
    tag = os.path.basename(vecfn)
    m = getdist.filterOutHeaders(tagToOrigMesg(tag), HEADERSTOSHOW)
    showMailDrawer(m, surf)
#     f = file(tag)
#     print tag
#     for l in f:
#         sys.stdout.write(l)
#     f.close()

TIMEOUTEV = USEREVENT + 1
gmousex = 0
gmousey = 0

def processEvent(e, surf, mu):
    global autoProceedF, nonDisplayMode, viewMappingMode, gMailShowing
    global gmousex, gmousey
    if quitEventP(e):
#        raise QuitMessage()
        raise demoutils.QuitException()
    if e.type == MOUSEBUTTONDOWN and e.button == 1:
        px, py = e.pos
        if gMailShowing:
            if not doFlash((px-icondx-icw)/MAG, (py-icondy-ich)/MAG, float(icw)/MAG, float(ich)/MAG, mu, surf):
                drawA(surf, mu, flashF=False)
                flip()
            gMailShowing = False
        else:
#            print px, py, icondx, icondy, (px-icondx-icw)/MAG, (py-icondy-ich)/MAG, float(icw)/MAG, float(ich)/MAG
            showMessageAt((px-icondx-icw)/MAG, (py-icondy-ich)/MAG, float(icw)/MAG, float(ich)/MAG, mu, surf)
    elif e.type == KEYDOWN and e.key == K_SPACE and not autoProceedF:
        proceed(surf, mu)
    elif e.type == KEYDOWN and e.key == K_t:
        autoProceedF = not autoProceedF
#     elif e.type == KEYDOWN and e.key == K_f:
#         pygame.display.toggle_fullscreen()
#     elif e.type == KEYDOWN and e.key == K_o:
#         s = axy3toSimpleSurf(m.M)
#         saveSurfAsPng(s, "som3.png")
#     elif e.type == KEYDOWN and e.key == K_m:
#         putdensityMatrix(getMappingDensityMatrix(m, gim))
#     elif e.type == KEYDOWN and e.key == K_d:
#         dumpMap(m, "som3.map")
    elif e.type == KEYDOWN and e.key == K_i:
        decay = mu.getDecay()
        print "decayLevel:", decay
        print "neighbourRadius:", mu.m.neighbourRadius(decay)
    elif e.type == KEYDOWN and e.key == K_n:
        nonDisplayMode = not nonDisplayMode
    elif e.type == KEYDOWN and e.key == K_v:
        viewMappingMode = not viewMappingMode
        drawA(surf, mu, flashF=False)
        flip()
#     elif e.type == KEYDOWN and e.key in xrange(K_1, K_9+1):
#         fn = "somdump%s.dat" % (e.key-K_1+1)
#         if e.mod & KMOD_SHIFT > 0:
#             mu.saveTo(fn)
#         else:
#             if not os.path.exists(fn):
#                 print "%s not found!" % fn
#             else:
#                 mu.loadFrom(fn)
#                 mu.staleMapping = True
# #                 mu.calcmappings()
#                 drawA(surf, mu)
#                 flip()
    elif e.type == KEYDOWN and e.key == K_RIGHT:
        raise demoutils.NextException()
    elif e.type == KEYDOWN and e.key == K_LEFT:
        raise demoutils.PrevException()
#     elif e.type == MOUSEMOTION and e.buttons[2] > 0:
    elif e.type == MOUSEMOTION:
        pygame.time.set_timer(TIMEOUTEV, 100)
        gmousex, gmousey = e.pos
    elif e.type == TIMEOUTEV and not gMailShowing:
        doFlash((gmousex-icondx-icw)/MAG, (gmousey-icondy-ich)/MAG, float(icw)/MAG, float(ich)/MAG, mu, surf)

def dumpMap(som, fn):
    f = file(fn, "w")
    f.write("%d rect %d %d gaussian\n" % (3, WIDTH, HEIGHT))
    for y in xrange(som.h):
        for x in xrange(som.w):
            f.write("%f %f %f\n" % tuple([256*v for v in som.M[x][y]]))
    f.close()

def putdensityMatrix(mat):
    for v in mat:
        for s in v:
            print "%4d" % s,
        print

def saveSurfAsPng(surf, fn):
    p = os.popen("convert tga:- %s" % fn, "w")
    pygame.image.save(surf, p)
    p.close()

def quitEventP(e):
    if e.type == QUIT:
        return True
    elif e.type == KEYDOWN and ( e.key == K_ESCAPE or e.key == K_q ):
        return True
    return False

respc = re.compile("[ \t]+")
def loadMap(m, mapfile):
    f = file(mapfile, "r")
    l = f.readline()
    for y in xrange(HEIGHT):
        for x in xrange(WIDTH):
            while True:
                l = f.readline().strip()
                if l[:1] != '#':
                    break
            a = respc.split(l)
            desti = m.M[x][y]
            for i in xrange(len(desti)):
                desti[i] = float(a[i])/256

def displayA(*args):
    somcore.displayInSurfArray(*args)

def displayX(*args):
    return pygame.surfarray.make_surface(*args)

def displayB(a, b, pos):
    a.blit(b, pos)
    
#     pygame.surfarray.blit_array(*args)

def displayC(scr, surf, pos):
    scr.blit(surf, pos)

def displayD():
    pygame.display.flip()

def cDisplay(surf, m, mesg=None, callback=None):
    displayA(m.M, m.SADISP, m.NM, DTHRESH, MAG)
#    somcore.displayInSurfArray(m.M, m.SADISP, m.NM, DTHRESH, MAG)
#    pygame.surfarray.blit_array(surf, m.SADISP)
    displayB(surf, m.SADISP)
    surf.fill((255, 0, 0, 255), pygame.Rect(m.latestWinner[0]*MAG, m.latestWinner[1]*MAG, MAG, MAG))
    scr = pygame.display.get_surface()
    displayC(scr, surf, (0, 0))
#     scr.blit(surf, (0, 0))
    dispCommon(scr, mesg, callback, m)
#     scr.fill(BLACK, STATUSRECT)
#     if mesg is not None:
#         s = FONT.render(mesg, 1, WHITE)
#         scr.blit(s, (STATUSRECT.x+2, STATUSRECT.y+2))

#     if callback is not None:
#         callback(scr, m)

#     displayD()

def cDraw(surf, mu, mesg=None, winner=None, flashF=False):
    m = mu.m
    if not nonDisplayMode:
        thresh = DTHRESH
        if not flashF:
            thresh = -1
        displayA(m.M, m.SADISP, m.NM, thresh, MAG)
        displayB(surf, displayX(m.SADISP), (0, 0))
        if winner is not None:
            mySetAt(surf, winner, RED)
        if viewMappingMode:
            mu.calcMappings()
            showMailIcons(surf, mu)

    drawCommon(surf, mesg, mu)

def pyDraw(surf, mu, mesg=None, winner=None, flashF=False):
    m = mu.m
    if not nonDisplayMode:
        for y in xrange(HEIGHT):
            for x in xrange(WIDTH):
                (r, g, b) = getcolor(m.M[x][y])
                if (x, y) == winner:
                    (r, g, b) = (255, 0, 0)
                elif flashF:
                    dist = m.NM[x][y]
                    if dist < DTHRESH:
                        rat = dist/DTHRESH
                        (r, g, b) = (int(255+(r-255)*rat), int(255+(g-255)*rat), int(255+(b-255)*rat))
                mySetAt(surf, (x, y), (r, g, b, 255))
        if viewMappingMode:
            mu.calcMappings()
            showMailIcons(surf, mu)

    drawCommon(surf, mesg, mu)

def drawCommon(surf, mesg, mu):
    surf.fill(BLACK, STATUSRECT)

    if mesg is not None:
        s = FONT.render(mesg, 1, WHITE)
        surf.blit(s, (STATUSRECT.x+2, STATUSRECT.y+2))

def makeTrans(surf, factor=0.7):
    for x in xrange(surf.get_width()):
        for y in xrange(surf.get_height()):
            pix = list(surf.get_at((x, y)))
            pix[3] = int(pix[3] * factor)
            surf.set_at((x, y), pix)

def mySetAt(surf, pos, col):
    (x, y) = pos
    surf.fill(col, pygame.Rect(x*MAG, y*MAG, MAG, MAG))

#draw = pyDraw
draw = cDraw

# def getcolor(v):
#    return (int(v[0]*256), int(v[1]*256), int(v[2]*256))
#    return [int(a*256) for a in v]

def fn2vec(fn):
    f = file(fn)
    rv = Numeric.array([float(x) for x in f], typecode='d')
    f.close()
    regularizeVec(rv)
    return rv

def regularizeVec(v):
    for i in xrange(len(v)):
        v[i] *= VECCOES[i]

def regularizeRandomVec(v):
    for i in xrange(len(v)):
        v[i] /= VECCOES[i]

def tagToOrigMesgFn(tag):
    return os.path.join(origMessageDir, tag)

def tagToOrigMesg(tag):
    f = codecs.open(tagToOrigMesgFn(tag), "r", "utf-8")
    rv = f.read()
    f.close()
    return rv    

mailFont = None
def getMailFont():
    global mailFont
    if mailFont is None:
        mailFont = pygame.font.Font(MAILFONTNAME, MAILFONTSIZE)
    return mailFont

def showMailDrawer(m, surf):
    s = pygame.Surface((950, 700))
    s.fill(WHITE)
    demoutils.renderText(s, m.split("\n"), x=20, y=20, font=getMailFont())
    s.set_alpha(220)
    surf.blit(s,
              (surf.get_width()/2.0-s.get_width()/2.0,
               surf.get_height()/2.0-s.get_height()/2.0))

def slide(fn, apF=False, ndF=False, vmF=True):
    global autoProceedF, nonDisplayMode, viewMappingMode

    initGlobals()

    autoProceedF = apF
    nonDisplayMode = ndF
    viewMappingMode = vmF

    mu = somuser.fromFile(fn)
    scr = pygame.display.get_surface()
    draw(scr, mu)
    flip()
    mainloop(scr, mu)

def initGlobals():
    global FONT

    if FONT is None:
        FONT = pygame.font.Font(None, 20)

def sommain(maxcnt=None):

    pygame.display.init()
    pygame.font.init()

    initGlobals()
#     surfw = WIDTH * MAG
#     surfh = HEIGHT * MAG
    scrw = WIDTH*MAG
    scrh = HEIGHT*MAG + STATH
    scr = pygame.display.set_mode((scrw, scrh))
#     surf = pygame.Surface((surfw, surfh))

    m = SOM()
    m.initNew(WIDTH, HEIGHT, len(VECCOES))

    for x in m.wr:
        for y in m.hr:
            regularizeRandomVec(m.M[x][y])

    mu = somuser(m, sys.argv[1:])

    # $B4{$K(B randomize $B$5$l$F$$$k$N$G!"L5BL$,$"$k(B
    if mapimg != None:
        loadMap(m, mapimg)

    draw(scr, mu)
    flip()
    mainloop(scr, mu, maxcnt)

def profmain():
    global autoProceedF

    autoProceedF = True
    sommain(gprofturns)

def main():
    if len(sys.argv) > 2 and sys.argv[1] == '-p':
        global gprofturns
        gprofturns = int(sys.argv[2])
        sys.argv.pop(1)
        sys.argv.pop(1)
        prof = hotshot.Profile("som3.prof")
        prof.runcall(profmain)
        prof.close()
    else:
        sommain()

if __name__ == '__main__':
    try:
        import som3
        som3.main()
    except demoutils.DemoException:
        pass
