# -*- coding: euc-jp -*-
import pygame
from pygame.locals import *
import sys
import codecs

DEFAULTWIDTH = 1024
DEFAULTHEIGHT = 768
DEFAULTFONTFILE = "gothic.ttf"

BLACK = pygame.color.Color("black")
WHITE = pygame.color.Color("white")

def slide(spec, font, surf=None):
    if surf is None:
        surf = pygame.display.get_surface()
    draw(surf, spec, font)

def renderText(font, specs, centering=False):
    surfaces = []
    ys = []

    w = 0
    h = 0
    color = WHITE
    for spec in specs:
        if type(spec) == tuple:
            color = spec
        elif type(spec) == pygame.font.Font:
            font = spec
        else:
            s = font.render(spec, 1, color)
            sw = s.get_width()
            sh = s.get_height()
            surfaces.append(s)
            ys.append(h)
            w = max(w, sw)
            h += sh

    rv = pygame.Surface((w, h))

    for i in range(len(surfaces)):
        if centering:
            x = (w-surfaces[i].get_width())/2
        else:
            x = 0
        rv.blit(surfaces[i], (x, ys[i]))
    return rv

def draw(surf, spec, font):
    surf.fill(BLACK)
    fs = renderText(font, spec, centering=True)
    w = surf.get_width()
    h = surf.get_height()
    sw = fs.get_width()
    sh = fs.get_height()
    surf.blit(fs, ((w-sw)/2, (h-sh)/2))
    pygame.display.flip()

def getspec():
    dec = codecs.getdecoder("euc-jp")
    return [dec(x)[0] for x in getspec0()]

def getspec0():
    if len(sys.argv) > 2 and sys.argv[1] == '-f':
        rv = [s.replace("\r", "").replace("\n", "") for s in file(sys.argv[2])]
    else:
        return [sys.argv[1]]

def main():
    spec = getspec()
    pygame.display.init()
    pygame.font.init()
    screen = pygame.display.set_mode((DEFAULTWIDTH, DEFAULTHEIGHT), FULLSCREEN)
    font = pygame.font.Font(DEFAULTFONTFILE, 200)
    draw(screen, spec, font)
    wait()

if __name__ == '__main__':
    main()
